/*
complex.h
*/

#ifndef _COMPLEX_H_
#define _COMPLEX_H_

#if defined(__GNUC__) && !defined(__NEC__) && !defined(__FUJITSU) && !defined(__CLANG_FUJITSU)
#pragma GCC diagnostic ignored "-Wunused-function"
#pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
#endif

#include <math.h>

// double

static inline d_complex_t d_complex(double r, double i)
{
	d_complex_t z;

	z.r = r;
	z.i = i;

	return z;
}

static inline d_complex_t d_add(d_complex_t a, d_complex_t b)
{
	d_complex_t z;

	z.r = a.r + b.r;
	z.i = a.i + b.i;

	return z;
}

static inline d_complex_t d_sub(d_complex_t a, d_complex_t b)
{
	d_complex_t z;

	z.r = a.r - b.r;
	z.i = a.i - b.i;

	return z;
}

static inline d_complex_t d_mul(d_complex_t a, d_complex_t b)
{
	d_complex_t z;

	z.r = (a.r * b.r) - (a.i * b.i);
	z.i = (a.r * b.i) + (a.i * b.r);

	return z;
}

static inline d_complex_t d_div(d_complex_t a, d_complex_t b)
{
	d_complex_t z;

	if ((fabs(b.r) <= 0) && (fabs(b.i) <= 0)) {
		z.r = z.i = 0;
	}
	else {
		z.r = ((a.r * b.r) + (a.i * b.i)) / ((b.r * b.r) + (b.i * b.i));
		z.i = ((a.i * b.r) - (a.r * b.i)) / ((b.r * b.r) + (b.i * b.i));
	}

	return z;
}

static inline d_complex_t d_rmul(double r, d_complex_t a)
{
	d_complex_t z;

	z.r = r * a.r;
	z.i = r * a.i;

	return z;
}

static inline d_complex_t d_add3(d_complex_t a, d_complex_t b, d_complex_t c)
{
	d_complex_t z;

	z.r = a.r + b.r + c.r;
	z.i = a.i + b.i + c.i;

	return z;
}

static inline d_complex_t d_add4(d_complex_t a, d_complex_t b, d_complex_t c, d_complex_t d)
{
	d_complex_t z;

	z.r = a.r + b.r + c.r + d.r;
	z.i = a.i + b.i + c.i + d.i;

	return z;
}

static inline double d_abs(d_complex_t z)
{
	return sqrt((z.r * z.r) + (z.i * z.i));
}

static inline double d_norm(d_complex_t z)
{
	return ((z.r * z.r) + (z.i * z.i));
}

static inline double d_rad(d_complex_t z)
{
	return atan2(z.i, z.r);
}

static inline double d_deg(d_complex_t z)
{
	return atan2(z.i, z.r) * 45.0 / atan(1.0);
}

static inline d_complex_t d_exp(double x)
{
	d_complex_t z;

	z.r = cos(x);
	z.i = sin(x);

	return z;
}

static inline d_complex_t d_sqrt(d_complex_t a)
{
	d_complex_t z;

	double r = sqrt(sqrt((a.r * a.r) + (a.i * a.i)));
	double p = 0.5 * atan2(a.i, a.r);
	z.r = r * cos(p);
	z.i = r * sin(p);

	return z;
}

static inline d_complex_t d_inv(d_complex_t a)
{
	d_complex_t z;

	if ((fabs(a.r) <= 0) && (fabs(a.i) <= 0)) {
		z.r = z.i = 0;
	}
	else {
		z.r = + a.r / ((a.r * a.r) + (a.i * a.i));
		z.i = - a.i / ((a.r * a.r) + (a.i * a.i));
	}

	return z;
}

static inline d_complex_t d_neg(d_complex_t a)
{
	d_complex_t z;

	z.r = -a.r;
	z.i = -a.i;

	return z;
}

// float

static inline f_complex_t f_complex(double r, double i)
{
	f_complex_t z;

	z.r = (float)r;
	z.i = (float)i;

	return z;
}

// cast

static inline d_complex_t d_cast(f_complex_t z)
{
	d_complex_t ret;

	ret.r = z.r;
	ret.i = z.i;

	return ret;
}

static inline f_complex_t f_cast(d_complex_t z)
{
	f_complex_t ret;

	ret.r = (float)z.r;
	ret.i = (float)z.i;

	return ret;
}

#endif  // _COMPLEX_H_
